
;                    -= Subs and Macros =-



;-----------------------------
; VGA registers and misc
;-----------------------------
 Sequencer      EQU     03C4h
 CRTC           EQU     03D4h
 Input1         EQU     03DAh
 Misc_Out       EQU     03C2h 
 Misc_In        EQU     03CCh
 Attrib_Out     EQU     03C0h
 Attrib_In      EQU     03C1h
 Graphic        EQU     03CEh

 Page_Size       =      21600
 X_Res           =      360
 X_Byte          =      360 / 4
 Y_Res           =      240
 Y_Byte          =      240
;-----------------------------




;                           *** MACROS ***


;-------------------------------------
; Draw one of the icons to the screen
;-------------------------------------
; In:   SI = Icon to draw
;       DI = Where to putt it
;       DS = ES = Screen
;       All 4 planes selected
;       Write mode 1
;
; Out:  Nothing
;
; Regs: CX SI DI
;--------------------------------------
Macro   Draw_Icon
	mov     cx,Icon_Res

Next_Line:
	movsb                           ; Move one line to visible screen
	movsb                           ; 3 * 4 = 12 (Icon resolution)
	movsb                           ; *
	add     di,X_Byte-(Icon_Res/4)
	loop    Next_Line
	
EndM    Draw_Icon



; Prepeare the status line (DS = ES = Data Segment)
Macro   Setup_Status
	xor     eax,eax
	cmp     [Byte ds:Finished],0
	jne     @@010
	movzx   ax,[Byte ds:D_Take]     ; Dimonds to take
	mov     di,Str_D_Take
	mov     cx,3
	call    Bin_To_ASCII
@@010:  movzx   ax,[Byte ds:D_Score]    ; What each dimond score
	mov     di,Str_D_Score
	mov     cx,3
	call    Bin_To_ASCII
	mov     ax,[Word ds:D_Count]    ; Dimonds we have taken
	mov     di,Str_D_Count
	mov     cx,3
	call    Bin_To_ASCII
	mov     ax,[Word ds:Time]       ; Time left
	shr     ax,4                    ; Divide by 16
	mov     di,Str_Time
	mov     cx,3
	call    Bin_To_ASCII
	mov     eax,[Dword ds:Score]    ; Our greate score :)
	mov     di,Str_Score
	mov     cx,6
	call    Bin_To_ASCII
EndM    Setup_Status



; Prepeare for writing text
Macro   Prepare_Text
	mov     dx,Sequencer            ; Select all 4 planes
	mov     ax,0F02h
	out     dx,ax
	mov     bl,2                    ; Set write mode 2
	call    Set_W_Mode
EndM    Prepare_Text



; Inn : AL = Direction
; Out : BP = Index to new position
Macro   Get_Index
	cmp     al,0
	je      @@010
	cmp     al,1
	je      @@020
	cmp     al,2
	je      @@030
	
	mov     bp,cx                   ; Up
	neg     bp
	jmp     @@099

@@010:  mov     bp,1                    ; Right
	jmp     @@099

@@020:  mov     bp,cx                   ; Down
	jmp     @@099

@@030:  mov     bp,-1                   ; Left

@@099:
EndM    Get_Index



; Get the index to the side of one position forward
; Inn   : AL = Direction
; Out   : BX = Left side
;       : BP = Right side
Macro   Get_Index2
	cmp     al,0
	je      @@110
	cmp     al,1
	je      @@120
	cmp     al,2
	je      @@130

	mov     bx,cx                   ; Up
	neg     bx
	mov     bp,bx
	inc     bx
	dec     bp
	jmp     @@199

@@110:  mov     bx,cx                   ; Right
	mov     bp,cx
	inc     bx
	neg     bp
	inc     bp
	jmp     @@199

@@120:  mov     bx,cx                   ; Down
	mov     bp,cx
	dec     bx
	inc     bp
	jmp     @@199

@@130:  mov     bx,cx                   ; Left
	mov     bp,cx
	neg     bx
	dec     bx
	dec     bp

@@199:
EndM    Get_Index2



; Get the index to the side of current position
; Inn   : AL = Direction
; Out   : BX = Left side
;       : BP = Right side
Macro   Get_Index3
	cmp     al,0
	je      @@210
	cmp     al,1
	je      @@220
	cmp     al,2
	je      @@230

	mov     bp,-1                   ; Up
	jmp     @@299

@@210:  mov     bp,cx                   ; Right
	neg     bp
	jmp     @@299

@@220:  mov     bp,1                    ; Down
	jmp     @@299

@@230:  mov     bp,cx                   ; Left

@@299:
EndM    Get_Index3





SEGMENT Code_Seg        Para    Public          'CODE'          USE16


ASSUME  CS:Code_Seg, SS:Stack_Seg





; Get one param (In uppercase)

; In:   CX = Param number
;
; Out:  SI = ASCIIZ string
;
; Regs: -

Proc    One_Param
	mov     si,Offset Param_Str
	dec     cx                      ; Zero based!
	jcxz    @@99                    ; CX = Zero --> SI = First param!

@@10:   lodsb                           ; Scan for zero terminator
	or      al,al
	jnz     @@10
	loop    @@10                    ; Do it CX times

@@99:   ret
EndP    One_Param




; Get the parameter string

; In:   ES = PSP
;
; Out:  BX = Number of parameters
;
; Regs: -

Proc    Get_Param
	mov     si,080h
	mov     di,Offset Param_Str
	xor     bx,bx                   ; Parameter count
	xor     ch,ch
	mov     ah,1

	mov     cl,[Byte es:si]         ; Get param length
	jcxz    @@99
	call    _UpperCase              ; Convert to uppercase
	inc     si

	xor     ah,ah                   ; Find first char

@@20:   mov     al,[Byte es:si]         ; Get char
	inc     si
	call    _Check                  ; Is it a seperator
	jne     @@30                    ; No, jump
	dec     cx                      ; Finished?
	jz      @@90
	or      ah,ah                   ; Have we added the zero?
	jz      @@20                    ; Yes, skipp it
	xor     ax,ax                   ; No, write it now
	inc     bl                      ; Param count
	inc     cx                      ; Don't DEC twice!
	jmp     @@31

@@30:   mov     ah,1                    ; Set new param flagg
@@31:   mov     [Byte ds:di],al         ; Write copy to string
	inc     di
	loop    @@20                    ; Do all chars in string

	mov     [Byte ds:di],0          ; Write last zero

@@90:   inc     bl                      ; Count last param
@@99:   ret
EndP    Get_Param



; ZF = 1 --> AL = Seperator (Jump Equal)
Proc    _Check
	cmp     al,020h                 ; Space?
	je      @@99
	cmp     al,09h                  ; TAB?

@@99:   ret
EndP    _Check




; Converts a string to uppercase
Proc    _UpperCase
	push    cx si
	inc     si

@@10:   mov     al,[Byte es:si]
	cmp     al,'a'
	jb      @@20
	cmp     al,'z'
	ja      @@20
	sub     al,'a'-'A'
	mov     [Byte es:si],al

@@20:   inc     si
	loop    @@10

@@99:   pop     si cx
	ret
EndP    _UpperCase




; SI = String --> CX = Length
Proc    Str_Length
	push    si
	mov     cx,0FFFFh               ; Max string length

@@10:   lodsb
	or      al,al
	loopnz  @@10

	not     cx
	dec     cx

	pop     si
	ret
EndP    Str_Length


; Moves one line down
Proc    New_Line
	mov     ah,2
	mov     dl,CR
	int     21h
	mov     dl,LF
	int     21h
	ret
EndP    New_Line



; SI = ASCIIZ string to write
Proc    Str_Write

	call    Str_Length              ; CX = Length

; CX = Str length
Str_Write2:
	mov     bx,1                    ; Standar out handle
	mov     dx,si
	mov     ah,040h                 ; Write to file or device
	int     21h

@@99:   ret
EndP    Str_Write







; Load an data file into memory

; In:   DX = File name
;       AX = Segment to load into
;
; Out:  CX = Size (para)
;       CY = 1 --> Error
;       AL = Error code
;       AH = -1 --> Out of memory
;
; Regs: ALL

Proc    Load_File
	mov     [Word ds:Curr_File],dx  ; Save file name

	push    ax                      ; Save seg
	mov     ax,03D00h               ; Open file (Read access)
	int     21h
	pop     ds                      ; DS = Where to load the file
	jc      @@99
	mov     bp,ax                   ; BP = Handle

; Read from file        
	xor     si,si
	mov     bx,ax                   ; Handle
	mov     cx,0FFF0h               ; Bytes to read
	xor     dx,dx                   ; DS:0

@@10:   mov     ah,03Fh                 ; Read FFF0h bytes
	int     21h
	jc      @@99
	cmp     ax,0FFF0h               ; Finished?
	jne     @@30                    ; Yes, return

	mov     ax,ds                   ; DS = DS + 0FFFh
	add     ax,0FFFh
	mov     ds,ax
	add     si,0FFFh                ; SI = File size
	cmp     si,[Word gs:Free_Mem]   ; Check if file is to big
	jb      @@10

	mov     ah,-1
	jmp     @@90


; Finish reading
@@30:   mov     bl,al
        shr     ax,4                    ; Size / 16
        test    bl,1111b                ; All ready para aligned?
        jz      @@40
        inc     ax                      ; No, para align

@@40:   add     si,ax
	mov     cx,si                   ; CX = File size (Para)
	sub     [Word gs:Free_Mem],cx

	mov     ah,03Eh                 ; Close file
	mov     bx,bp
	int     21h

@@99:   mov     ah,0                    ; Don't change flaggs!
@@90:   mov     bx,Data_Seg
	mov     ds,bx
	ret
EndP    Load_File





;-------------------------------------
; Do the moving on the virtual map
;-------------------------------------
; In:   Nothing
;
; Out:  Nothing
;
; Regs: -
;--------------------------------------
Proc    Move_Map
	push    ds                      ; DS = ES = Data Seg
	pop     es

	mov     si,Offset Map1          ; Setup SI and DI to right Map
	mov     di,Offset Map2
	cmp     [Byte ds:Curr_Map],0
	je      @@10
	xchg    si,di


; Copy old map to new map
@@10:   push    si di
	mov     cx,[Word ds:Map_Size]
	shr     cx,2                    ; Convert to Dword move
	inc     cx                      ; One extra
	rep     movsd
	pop     di si

	mov     cx,[Word ds:Map_Size]   ; Bytes to move
	movzx   bx,[Byte ds:Map_H]      ; Bytes per line
	xor     dh,dh                   ; DH = 0


; Handle each icon
@@20:   movzx   eax,[Byte ds:si]
	cmp     al,[Byte ds:di]
	jne     Return
	and     al,00011111b            ; Mask out controll bits
	sub     al,4                    ; Skipp icon 0 - 3
	js      Return
	cmp     al,Num_Icons-4-1        ; Number to large?
	ja      Map_Error               ; Yes, Error :(
	jmp     [Word ds:Action_Table+eax*2]

; Icon dosen't need an own sub
Return: inc     si
	inc     di
	loop    @@20

@@99:   ret
EndP    Move_Map


INCLUDE         "ACTION.INC"


;-------------------------------------
; Draw the icons to the screen
;-------------------------------------
; In:   BP = What to draw
;       DI = Where to draw it
;
; Out:  Nothing
;
; Regs: -
;--------------------------------------
Proc    Draw_Screen
	mov     dx,Sequencer            ; Select all 4 planes
	mov     ax,0F02h
	out     dx,ax
	mov     bl,1                    ; Set write mode 1
	call    Set_W_Mode
	push    es                      ; DS = ES = Screen
	pop     ds

; Calculate where to start drawing in the map
	mov     al,[Byte gs:Pos_V]
	mul     [Byte gs:Map_H]
	add     al,[Byte gs:Pos_H]
	adc     ah,0
	add     bp,ax

; Draw the screen 30 X 19
	xor     bh,bh                   ; BX = BL
	mov     ax,19                   ; 19 lines of icons

Draw_Screen2:
	mov     dx,30                   ; 30 icons on each line

Draw_Line:
	mov     bl,[Byte gs:bp]         ; Get one byte from the map
	and     bl,00011111b            ; Mask out controll bit
	inc     bp                      ; Next icon in map
	add     bl,bl                   ; It's a word table
	mov     si,[Word gs:Icon_Table + bx]    ; Get offset to icon

; Change frames for animation
	or      bl,bl                   ; 'Black'
	jz      @@13
	cmp     bl,04*2                 ; Special brick wall?
	je      @@19
	cmp     bl,05*2
	je      @@19
	cmp     bl,07*2                 ; Mask out the icons with no
	jb      @@20                    ; animation
	je      @@12                    ; Green wierd stuff?
	cmp     bl,20*2                 ; Home?
	je      @@16
	cmp     bl,15*2
	ja      @@20
	cmp     bl,09*2                 ; Dimond?
	je      @@10
	cmp     bl,12*2                 ; Monster 1?
	je      @@14

; Icons with 2 frames
@@11:   cmp     [Byte gs:Frame1],0
	je      @@20
	add     si,Icon_size/4          ; Frame 2
	jmp     @@20

; The green wierd stuff
@@12:   cmp     [Byte gs:Frame3],2
	jb      @@20
	add     si,Icon_Size/4
	jmp     @@20

; Monster 1
@@14:   cmp     [Byte gs:Frame4],0
	je      @@20
	add     si,Icon_Size/4
	cmp     [Byte gs:Frame4],1
	je      @@20
	add     si,Icon_Size/4
	jmp     @@20

; Our home
@@16:   mov     cl,[Byte gs:Finished]
	and     cl,01111111b
	cmp     cl,1                    ; Shall home flash?
	jae     @@17                    ; Yes, do animation
@@18:   mov     si,Icon_Offset + ((Icon_Size/4)*2)  ; Else, draw normal wall
	jmp     @@20
@@17:   cmp     [Byte gs:Frame3],1      ; Do animation
	jbe     @@20
	jmp     @@18


; The special brick wall
@@19:   cmp     [Byte gs:Wall_Status],1 ; Active?
	je      @@11                    ; Yes, change frame
	mov     si,Icon_Offset + ((Icon_Size/4)*3)
	jmp     @@20                    ; No, write normal brick wall


; "Black"
@@13:   cmp     [Byte gs:Extra_Life],bl ; BL = Zero... Optimiaztion :)
	jz      @@20
	add     si,(Icon_Size/4)*28
	jmp     @@20


; Icons with 3 frames
@@10:   cmp     [Byte gs:Frame2],0
	je      @@20
	add     si,Icon_Size/4          ; Frame 2
	cmp     [Byte gs:Frame2],1
	je      @@20
	add     si,Icon_Size/4          ; Frame 3

@@20:   Draw_Icon

	sub     di,(X_Byte*12)-3        ; DI = Place to draw next icon
	dec     dx
	jnz     Draw_Line

	add     di,(X_Byte*11)          ; DI = Next line
	mov     bl,[Byte gs:Map_H]
	sub     bl,30
	add     bp,bx                   ; BP = Offset to next line
	dec     ax
	jnz     Draw_Screen2

	mov     ax,Data_Seg             ; Restore DS
	mov     ds,ax
	xor     bl,bl
	call    Set_W_Mode

	cmp     [Byte ds:Finished],1    ; Shall we flash the screen?
	je      @@99
	ret

; Flash up the screen
@@99:   mov     dx,03C8h                ; White
	xor     al,al
	out     dx,al
	inc     dx
	mov     al,63
	out     dx,al
	out     dx,al
	out     dx,al

	mov     cx,5
Delay2: call    Wait_VR
	loop    Delay2

	mov     dx,03C8h                ; Black again
	xor     al,al
	out     dx,al
	inc     dx
	out     dx,al
	out     dx,al
	out     dx,al
	inc     [Byte ds:Finished]

	ret
EndP    Draw_Screen



;-------------------------------------
; Convert a binary value to ASCII
;-------------------------------------
; In:   EAX = Value to convert
;       DI = String to hold result
;       CX = Digits in string
;
; Out:  Nothing
;
; Regs: ALL - BP
;--------------------------------------
Proc    Bin_To_ASCII
	xor     si,si
	mov     ebx,10                  ; Convert to decimal (Base 10)

@@10:   xor     edx,edx                 ; EDX:EAX = EAX
	div     ebx                     ; EAX = Answer, EDX <- Remainder
	or      dl,'0'                  ; Convert digit to ASCII 0 - 9
	push    dx                      ; Save digit
	inc     si                      ; Count digit
	loop    @@10

	mov     cx,si                   ; CX = Digit on stack
@@20:   pop     ax                      ; al <-- Digit
	stosb
	loop    @@20
	ret
EndP    Bin_To_ASCII




;-------------------------------------
; Install interupt handlers
;-------------------------------------
; In:   Nothing
;
; Out:  Nothing
;
; Regs: -
;--------------------------------------
Proc    Setup_Int
	cli

; Set timer 0 to normal speed (Some one may have changed it)
	mov     al,36h
	out     43h,al
	IODELAY
	xor     ax,ax
	out     40h,al
	IODELAY
	mov     al,ah
	out     40h,al

; Install Keyboard Handler
	mov     eax,[Dword fs:09h*4]
	mov     [Dword cs:Int_09h],eax
	mov     [Word fs:09h*4],offset Keyb_Int
	mov     [Word fs:(09h*4)+2],cs

; Install Timer Interupt
	mov     eax,[Dword fs:08h*4]
	mov     [Dword cs:Int_08h],eax
	mov     [Word fs:08h*4],Offset Timer_Int
	mov     [Word fs:(08h*4)+2],cs


; Install CTRL-BREAK and Critical error handler
        mov     [Word fs:23h*4],Offset Err1_Int
        mov     [Word fs:(23h*4)+2],cs
        mov     [Word fs:24h*4],Offset Err2_Int
        mov     [Word fs:(24h*4)+2],cs

	sti
	ret
EndP    Setup_Int



;-------------------------------------
; Set Mode X  (360 X 240 , 256 color)
;-------------------------------------
; In:   Nothing
;
; Out:  ES:DI = Screen
;       All 4 planes selected
;
; Regs: Unknown :)
;--------------------------------------
Proc    ModeX
	mov     ax,0093h                ; Use BIOS to set standard mode 13h
	int     10h                     ; And don't clear VGA mem
	
	mov     dx,Sequencer
	mov     ax,0604h                ; Unchain my VGA
	out     dx,ax                   ; *
	mov     ax,0F02h                ; Select all 4 planes
	out     dx,ax                   ; *
	
	mov     ax,0100h                ; Synchronous reset since we're
	out     dx,ax                   ; changing Clocking Mode reg

	mov     dx,Misc_Out             ; 480 scan lines and 28 MHz clock
	mov     al,11100111b            ; *
	out     dx,al                   ; *
	
	mov     dx,Sequencer            ; Reset sequencer
	mov     ax,0300h
	out     dx,ax

	mov     dx,CRTC                 ; Turn of write protect
	mov     al,011h
	out     dx,al
	inc     dx
	in      al,dx
	and     al,01111111b
	out     dx,al
	dec     dx                      ; DX = CRTC

	mov     si,offset VGA_Registers ; Setup CRT registers
	mov     cx,Num_Regs

Setup_Regs:
	lodsw
	out     dx,ax
	loop    Setup_Regs

	mov     ax,0A000h               ; ES:DI = Screen
	mov     es,ax                   ; *
	xor     di,di                   ; *

	xor     eax,eax                 ; Clear all VGA mem
	mov     cx,04000h
	rep     stosd
	ret                             ; DI is now zero
EndP    ModeX



;-------------------------------------
; Do the startup stuff
;-------------------------------------
; In:   Nothing
;
; Out:  Nothing
;
; Regs: -
;--------------------------------------
Proc    Init

; Clear global data
        mov     si,Offset Rot_Count
	mov     cx,Global_Clear
	xor     al,al
@@10:   mov     [Byte ds:si],al
	inc     si
	loop    @@10

; Move icons to VGA memory for fast accses
	mov     dx,Sequencer
	mov     si,offset Icons
	mov     di,Icon_Offset
	mov     bx,Num_Icons
	
Next_Icon:
	mov     ax,0102h                ; AL = Map mask reg, AH = Plane 1
	mov     bp,di                   ; Save DI

Next_Plane:
	out     dx,ax                   ; Select plane in AH
	mov     cx,(Icon_Size/16)
	rep     movsd                   ; Move one plane of one icon
	mov     di,bp                   ; Restore DI

	shl     ah,1                    ; Next plane
	test    ah,10000b               ; Done all 4 planes?
	jz      Next_Plane              ; No, next plane
	
	add     di,(Icon_Size/4)        ; Next icon
	dec     bx                      ; Repeat all icons
	jnz     Next_Icon               ; *

	ret
EndP    Init



;-------------------------------------
; Things to do before exit
;-------------------------------------
; In:   Nothing
;
; Out:  Nothing
;
; Regs: -
;--------------------------------------
Proc    Exit_Stuff
	cli

; Unistall interupt handlers
	mov     eax,[Dword cs:Int_09h]  ; Restore old Keyboard Handler
	mov     [Dword fs:09h*4],eax
	mov     eax,[Dword cs:Int_08h]  ; Restore old timer interupt
	mov     [Dword fs:08h*4],eax

	sti


; Close file if exist
	mov     bx,[Word ds:High_Handle]
	or      bx,bx
	jz      @@99
	mov     ah,03Eh
	int     21h                     ; Ignore errors

@@99:   ret
EndP    Exit_Stuff



;-------------------------------------
; Write a text string in Mode X
;-------------------------------------
; In:   BP = ASCIIZ Text to write
;       DI = Where to write it
;
; Out:  Nothing
;
; Regs: ALL
;--------------------------------------
Proc    Write_Text
	mov     dx,Graphic              ; Write color to Bit Mask register
	mov     al,[Byte es:0FFFFh]     ; Dummy read to clear the VGA latches


; Clear old text/whatever
	push    di
	mov     cx,(X_Byte * 12/4) - 1
	xor     eax,eax
	rep     stosd
	pop     di

	
; Write text        
	movzx   bx,[Byte ds:Char_Space] ; Space between chars
	neg     bx
	add     bx,X_Byte*12            ; BX = X_Byte*12 - Space


; Ready to write string
@@10:   mov     si,Offset Font
@@15:   movzx   ax,[Byte ds:bp]         ; Get char from string
	or      al,al                   ; Shall we change color?
	jnz     @@20
	mov     ah,[Byte ds:bp+1]       ; Get color
	or      ah,ah                   ; Shall we exit?
	jz      @@99                    ; Exit
	mov     al,08h                  ; Write color to Bit Mask Register
	out     dx,ax
	inc     bp
	inc     bp
	jmp     @@15

@@20:   sub     ax,32                   ; Convert to my font
	inc     bp
	shl     ax,4                    ; Multiply by 16
	add     si,ax                   ; SI = Offset to char's bit pattern
	mov     cx,12                   ; Write 12 Lines

; Write one char
@@30:   lodsb                           ; Get first line (8 pixels)
	stosb                           ; Write first 4 pixels
	shr     al,4
	stosb                           ; Write last 4 pixels
	add     di,X_Byte-2             ; Next line
	loop    @@30

	sub     di,bx                   ; Next chars possition
	jmp     @@10

; Finish writing
@@99:   ret
EndP    Write_Text



; Set the write mode
; BL = Mode (0-3)
Proc    Set_W_Mode
	mov     dx,Graphic
	mov     al,05h                  ; Select Mode Register
	out     dx,al                   ; *
	IODELAY
	inc     dx                      ; Read the Mode Register
	in      al,dx                   ; *
	IODELAY
	and     al,11111100b            ; Set write mode
	or      al,bl                   ; *
	out     dx,al                   ; *

	ret
EndP    Set_W_Mode



; Clear the screen and set start address to 0
Proc    Clear_Screen
	mov     ax,0A000h
	mov     es,ax

	mov     dx,Sequencer            ; Select all 4 planes
	mov     ax,0F02h
	out     dx,ax

	call    Wait_VR
	mov     cx,Page_Size / 4        ; Clear screen
	xor     di,di
	xor     eax,eax
	rep     stosd

	xor     bx,bx                   ; Set page 1
	call    Write_SA

	xor     bl,bl                   ; Set normal write mode
	call    Set_W_Mode

	mov     dx,Graphic              ; Reset Bit Mask register
	mov     ax,0FF08h
	out     dx,ax

	ret
EndP    Clear_Screen




;-------------------------------------
; Loads up to 256*3 collor registers
;-------------------------------------
; In:   BL = First register
;       CX = Register to load (*3)
;       SI = Values to load (DS:SI)
;
; Out:  Nothing
;
; Regs: AX CX DX SI
;--------------------------------------
Proc    Write_Col
	call    Wait_VR

Write_Col2:
	mov     dx,03C8h                ; PEL address register
	mov     al,bl                   ; First reg
	out     dx,al
	inc     dx                      ; Select the data register
	rep     outsb

	ret
EndP    Write_Col



;-------------------------------------
; Write the Start Address register
;-------------------------------------
; In:   BX = Start address
;
; Out:  DX = CRTC
;
; Regs: AX DX
;-------------------------------------
Proc    Write_SA
	mov     dx,Input1
@@10:   in      al,dx                   ; Wait for VR
	test    al,1000b                ; *
	jz      @@10                    ; *

	cmp     [Byte ds:Playing],0     ; Only update if we are playing
	je      @@19

; Update palette
	mov     dx,03C8h
	mov     al,33                   ; Set color 33
	out     dx,al
	inc     dx
	mov     si,Offset Pal+(33*3)
	mov     cx,1*3
@@15:   mov     al,[Byte gs:si]
	inc     si
	out     dx,al
	loop    @@15

@@19:   mov     dx,Input1
@@20:   in      al,dx                   ; Wait for none VR
	test    al,1000b                ; *
	jnz     @@20                    ; *

Write_SA2:
	mov     dx,CRTC
	mov     al,0Ch                  ; Start address high
	mov     ah,bh                   ; AH = Value
	out     dx,ax

	inc     al                      ; Start address low
	mov     ah,bl                   ; AH = Value
	out     dx,ax
	
	ret
EndP    Write_SA



;-------------------------------------
; Wait for the start of VR
;-------------------------------------
; In:   Nothing
;
; Out:  Nothing
;
; Regs: AX DX
;-------------------------------------
Proc    Wait_VR
	mov     dx,Input1
@@10:   in      al,dx                   ; Wait for non VR
	test    al,1000b                ; *
	jnz     @@10                    ; *

@@20:   in      al,dx                   ; Wait for VR
	test    al,1000b                ; *
	jz      @@20                    ; *
	ret
EndP    Wait_VR






;---------------------------------------------
; Draws a transparent picture on the screen
;---------------------------------------------
; In:   ES:EDI = Screen location (Linear)
;       DS = Data Seg
;       SI = Picture
;            First word = (Hor Res / 4)
;            Second word = Vert Res
;
; Out:  Nothing
;
; Regs: ALL!
;---------------------------------------------
Proc    TD_Draw_BMP
	mov     cx,[Word ds:si]         ; Get hor res / 4
	mov     dx,[Word ds:si+2]       ; Get vert res
	mov     [Word ds:Call_Hor],cx   ; Save resolution
	mov     [Word ds:Call_Vert],dx  ; *
	mov     bp,X_Byte               ; BP = Bytes to next line
	sub     bp,cx                   ; *
	add     si,4                    ; SI = Picture data

	mov     bx,di                   ; Get position
	and     bx,0011b                ; Only want the 2 lower bits
	mov     ah,[Byte ds:Table1+bx]  ; BH = First map mask
	
	shr     edi,2                   ; Convert to Mode X format
	mov     [Word ds:Call_DI],di    ; Save DI
	
@@Next_Plane:
	mov     dx,Sequencer            ; Select plane in BH
	mov     al,2                    ; *
	out     dx,ax                   ; *
	
	mov     cx,[Word ds:Call_Hor]   ; Restore registers
	mov     dx,[Word ds:Call_Vert]  ; *
	mov     di,[Word ds:Call_DI]    ; *

@@Draw_Plane:
	push    cx
@@Draw_Line:
	lodsb                           ; Get pixel color
	or      al,al                   ; Is pixel color = Background
	jz      @@10                    ; Yes, don't write it
	add     al,[Byte ds:Move_Color]
	mov     [Byte es:di],al         ; No , write it
@@10:   inc     di                      ; Next pixel
	loop    @@Draw_Line             ; Repeat whole line
	pop     cx
	add     di,bp                   ; Calclulate bytes to next line
	dec     dx
	jnz     @@Draw_Plane

	shl     ah,1                    ; Next plane
	test    ah,10000b
	jz      @@Next_Plane

; Do last byte
	inc     [Word ds:Call_DI]       ; Move one byte forward
	mov     ah,0001b                ; Start with plane one

@@Next_Plane2:
	cmp     ah,[Byte ds:Table1+bx]  ; Finished ?
	je      @@99

	mov     dx,Sequencer            ; Select plane in AH
	mov     al,2                    ; *
	out     dx,ax                   ; *
	
	mov     cx,[Word ds:Call_Hor]   ; Restore registers
	mov     dx,[Word ds:Call_Vert]  ; *
	mov     di,[Word ds:Call_DI]    ; *

@@Draw_Plane2:
	push    cx
Draw_Line2:
	lodsb                           ; Get pixel color
	or      al,al                   ; Is pixel color = Background
	jz      @@20                    ; Yes, don't write it
	add     al,[Byte ds:Move_Color]
	mov     [Byte es:di],al         ; No , write it
@@20:   inc     di                      ; Next pixel
	loop    Draw_Line2              ; Repeat whole line
	pop     cx
	add     di,bp                   ; Calclulate bytes to next line
	dec     dx
	jnz     @@Draw_Plane2

	shl     ah,1                    ; Next plane
	jmp     @@Next_Plane2

@@99:   ret
EndP    TD_Draw_BMP

EndS    Code_Seg



SEGMENT Data_Seg        Para    Public          'DATA'          USE16

VGA_Registers   dw      06B00h          ; Horz total
		dw      05901h          ; Horz Displayed
		dw      05A02h          ; Start Horz Blanking
		dw      08E03h          ; End Horz Blanking
		dw      05E04h          ; Start Horz Sync
		dw      08A05h          ; End Horz Sync
		
		dw      00D06h          ; Vertical Total
		dw      03E07h          ; Overflow
		dw      0EA10h          ; V Sync Start
		dw      08C11h          ; V Sync End and protection
		dw      0DF12h          ; Vertical Displayed
		dw      0E715h          ; V Blank Start
		dw      00616h          ; V Blank End

		dw      04109h          ; Cell Height (2 Scan Lines)
		dw      00014h          ; Dword Mode off
		dw      0E317h          ; Byte Mode on
		dw      02D13h          ; Offset register 360 / 8 = 45 (02Dh)
Num_Regs        EQU     17


Table1          db      0001b           ; Table for writing a BMP picture
		db      0010b
		db      0100b
		db      1000b


; Temperorary variables
Call_DI         dw      ?
Call_Hor        dw      ?
Call_Vert       dw      ?
Move_Color      db      ?

EndS     Data_Seg
