
;     *** Menu screen ***

; In:   AL = 0 -> Finished!
;       AL = 1 -> Game over
;       AL = 2 -> Goto menu
;       AL = -1 -> First time playing
;
; Out:  AL = 0 -> Play level
;       [Num_Level] = Level
;       AL = 1 -> Exit to DOS
;
; Regs: ALL!

Proc    Menu
        mov      [Byte ds:Menu_Active],1; Activate

        push    ax
        xor     al,al
        mov     [Byte ds:Last_Key],al   ; Reset variables
        mov     [Byte ds:ESC_Flagg],al
        mov     [Byte ds:Level_Num],1

        call    Clear_Screen            ; Reset screen

        mov     cx,256*3                ; Setup menu palete
        mov     si,offset Pal2
        xor     bl,bl
        call    Write_Col               ; Now in VR!
        pop     ax

        or      al,al                   ; Finished?
        jnz     @@02

;
; # Finished all levels!
;
        Prepare_Text                    ; Write message
        mov     [Byte ds:Char_Space],2
        mov     bp,Offset Str_Finished
        mov     di,X_Byte*30 + 28
        call    Write_Text

        movzx   eax,[Byte ds:Men]       ; give an extra 1.500 for each
        imul    eax,1500                ; life left
        add     [Dword ds:Score],eax

        mov     al,1

@@02:   cmp     al,1
        jne     @@00                    ; Update higscore list?
        call    High_Score
        call    Save_High               ; Don't return on error

Return2:                                ; Return from view high score
@@00:   call    Set_BG                  ; Draw 'static' background

        mov     edi,(360*29)+67         ; Draw BD Logo
        mov     si,Offset BD_Logo
        mov     [Byte ds:Move_Color],15
        call    TD_Draw_BMP

        mov     edi,(360*160)+23        ; Draw SS Logo
        mov     si,Offset SS_Logo
        mov     [Byte ds:Move_Color],139
        call    TD_Draw_BMP

        mov     si,Offset High_List+4   ; Setup high score name
        mov     di,Menu_Name
        mov     cx,10
@@15:   mov     al,[Byte ds:si]
        inc     si
        mov     [Byte ds:di],al
        inc     di
        loop    @@15


; *** Menu loop ***
@@20:   call    _Keys                   ; Do keyboard input

        push    es
        push    ds                      ; ES = Data
        pop     es

        mov     di,Menu_Level           ; Setup level number
        movzx   eax,[Byte ds:Level_Num]
        mov     cx,2
        call    Bin_To_ASCII

        mov     si,offset Pal_Rot       ; Update palette
        mov     bl,3
        mov     cx,4*3
        call    Write_Col               ; Now in VR!

        pop     es                      ; Write string
        Prepare_Text
        mov     [Byte ds:Char_Space],2
        mov     bp,Offset Menu_Str
        mov     di,X_Byte*220 + 2
        call    Write_Text

        mov     al,-1                   ; Assume exit
        cmp     [Byte ds:ESC_Flagg],0
        jne     @@30

        cmp     [Byte ds:Last_Key],01Ch ; Wait for Enter
        jne     @@20
        xor     al,al                   ; Play game

@@30:   push    ax
        call    Clear_Screen            ; Clear screen when changing palette
        pop     ax
        mov     [Byte ds:Menu_Active],0 ; Deactivate, and return...
        ret
EndP    Menu




; Update the high score list

Proc    High_Score
        inc     [Byte ds:BIOS_Keyb]     ; Activate BIOS keyboard handler

        call    Wait_VR

; Draw high score list
        Prepare_Text
        mov     [Byte ds:Char_Space],2

        mov     bp,Offset High1_Str     ; Write title line
        mov     di,X_Byte*70 + 28
        call    Write_Text

        xor     bl,bl                   ; Draw sepperator line
        call    Set_W_Mode
        mov     di,(X_Byte*85)+28
        mov     al,1
        mov     dx,2
@@15:   mov     cx,(16*8)/4
        rep     stosb
        add     di,X_Byte - (16*8)/4
        dec     dx
        jnz     @@15


; Search for high score        
        mov     eax,[Dword ds:Score]
        mov     si,Offset High_List
        mov     cx,6
@@50:   cmp     [Dword ds:si],eax       ; Higher score?
        jb      @@60
        add     si,14
        loop    @@50
        xor     si,si
        jmp     @@90


; Found high score. SI = Offset
@@60:   add     si,4
        push    si

; Move list down (CX = Entrys left)
        dec     cx                      ; Move one less
        jz      @@66
        mov     dx,cx
        mov     di,Offset High_List+(14*5) ; Last entry
        mov     si,Offset High_List+(14*4)

@@62:   mov     cx,14                   ; Copy one entry
@@65:   mov     al,[Byte ds:si]
        inc     si
        mov     [Byte ds:di],al
        inc     di
        loop    @@65
        sub     si,14*2
        sub     di,14*2
        dec     dx
        jnz     @@62

@@66:   pop     si
        mov     di,si                   ; Clear old name
        mov     cx,10
@@68:   mov     [Byte ds:di],' '
        inc     di
        loop    @@68

        mov     eax,[Dword ds:Score]
        mov     [Dword ds:si-4],eax     ; Write new score
        xor     cx,cx                   ; Char count


; *** High score loop ***
@@90:   push    cx si                   ; Save high score data

        push    es
        push    ds                      ; ES = Data seg
        pop     es


; Setup high score list
        mov     si,Offset High_List
        mov     di,Offset Map1          ; No one use this now!
        mov     bp,6

@@10:   mov     [Byte ds:di],0          ; Score color
        mov     [Byte ds:di+1],27-18
        add     di,2
        mov     eax,[Dword ds:si]       ; Get score
        mov     cx,6                    ; 6 digits
        push    cx si di
        call    Bin_To_ASCII            ; Convert to ASCII
        pop     di si cx

        mov     [Byte ds:di+6],' '      ; Add space
        mov     [Byte ds:di+7],'-'
        mov     [Byte ds:di+8],' '

        mov     [Byte ds:di+9],0        ; Name collor
        mov     [Byte ds:di+10],29-18
        add     di,11                   ; Name
        add     si,4                    ; Name
        mov     cx,10
        rep     movsb                   ; Copy name
        mov     [Word ds:di],0          ; Add string terminator
        add     di,2
        dec     bp                      ; Do all 6 names
        jnz     @@10


; Write names and score
        pop     es                      ; ES = Screen
        call    Wait_VR
        Prepare_Text
        mov     bp,Offset Map1
        mov     di,X_Byte*100 + 25
        mov     cx,6
@@20:   push    cx di bp
        call    Write_Text
        pop     bp di cx
        add     bp,25
        add     di,X_Byte*12
        loop    @@20
        
        pop     si cx

        or      si,si                   ; Shall we draw cursor?
        jz      @@25
        cmp     cx,10
        je      @@25
        mov     al,123                  ; Flash currsor
        cmp     [Byte ds:Currsor],0
        je      @@22
        mov     al,' '
@@22:   mov     [Byte ds:si],al         ; Draw cursor

@@25:   call    _Keys2                  ; SI = High score name
        or      al,al                   ; Run until ESC is pressed
        jnz     @@90

        mov     [Byte ds:si],' '        ; Delete currsor
        dec     [Byte ds:BIOS_Keyb]     ; Deactivate BIOS handler
        call    Clear_Screen
        ret
EndP    High_Score




; SI = High score name, CX = Char count

Proc    _Keys2
        mov     ah,1                    ; Check for a key
        int     16h
        jz      @@99
        xor     ah,ah                   ; Get key
        int     16h

        cmp     ah,1                    ; ESC?
        je      @@ESC

        or      si,si                   ; High score?
        jz      @@99

        cmp     al,08h                  ; Back space
        jne     @@10
        or      cx,cx
        jz      @@99
        dec     cx                      ; Delete char
        dec     si
        mov     [Byte ds:si],' '
        cmp     cx,9                    ; Last char
        je      @@99
        mov     [Byte ds:si+1],' '      ; Delete old currsor
        jmp     @@99


@@10:   cmp     al,CR                   ; Enter?
        jne     @@20
        cmp     cx,10                   ; Beond last char?
        je      @@15
        mov     [Byte ds:si],' '        ; Clear currsor
@@15:   xor     si,si                   ; Finished edit
        jmp     @@99


; AL = Key
@@20:   cmp     al,32                   ; Filter out unwantet keys
        jb      @@99
        cmp     al,125
        ja      @@99

        cmp     cx,10                   ; Max 10 chars
        je      @@99
        mov     [Byte ds:si],al         ; Write key
        inc     cx
        inc     si

@@99:   mov     al,1
        ret

@@ESC:  xor     al,al
        ret
EndP    _Keys2




; Do the keyboard innput (Menu)

Proc    _Keys
        mov     al,[Byte ds:Last_Key]   ; Get key


; Check for + and -
        mov     ah,[Byte ds:Level_Num]  ; AH = Level number

        cmp     al,04Ah                 ; -
        jne     @@10
        sub     ah,4
        js      @@99
        jmp     @@19

@@10:   cmp     al,04Eh                 ; +
        jne     @@20
        add     ah,4
        cmp     ah,[Byte ds:Num_Level]  ; Max level
        ja      @@99
@@19:   mov     [Byte ds:Level_Num],ah
        jmp     @@99


@@20:   cmp     al,023h                 ; 'H'
        jne     @@30
        add     sp,2                    ; Fixup stack
        call    Clear_Screen
        mov     [Dword ds:Score],0
        call    High_Score
        call    Clear_Screen
        mov     [Byte ds:Last_Key],0
        jmp     Return2


; Futere keys...
@@30:   ;jmp     @@99

@@99:   mov     [Byte ds:Last_Key],0    ; Remove key
        ret
EndP    _Keys




; Count the number of levels in the game file

Proc    Count_Levels
        push    es
        mov     ax,[Word ds:Level_Seg]
        mov     es,ax
        mov     di,4                    ; ES:DI = Level info header

        xor     al,al                   ; Level count
@@10:   mov     bx,[Word es:di]
        cmp     bx,-1                   ; Last level?
        je      @@20
        inc     al
        inc     di
        inc     di
        jmp     @@10

@@20:   mov     [Byte ds:Num_Level],al  ; Save
        pop     es
        ret
EndP    Count_Levels




; Setup the background data

Proc    Set_BG

; Draw window
        mov     dx,Sequencer            ; Select all 4 planes
        mov     ax,0F02h
        out     dx,ax
        mov     di,(X_Byte*10)+28/4     ; Screen position

        mov     al,1                    ; Draw the first 4 lines
        mov     dx,4
@@10:   mov     cx,76
        rep     stosb
        add     di,14
        dec     dx
        jnz     @@10

        mov     dx,130-8                ; Draw the middle lines
@@20:   mov     al,1
        stosb
        mov     al,2
        mov     cx,76-2
        rep     stosb
        mov     al,1
        stosb
        add     di,14
        dec     dx
        jnz     @@20

        mov     al,1                    ; Draw last 4 lines
        mov     dx,4
@@30:   mov     cx,76
        rep     stosb
        add     di,14
        dec     dx
        jnz     @@30


; Draw the static data
        mov     dx,Sequencer            ; Select the 1 plane
        mov     ax,0102h
        out     dx,ax
        mov     di,(X_Byte*14)+32/4     ; Screen position

        mov     ah,3                    ; Starting collor
        mov     dx,76-2                 ; Collumns to draw

@@40:   mov     al,ah                   ; Draw it!
        mov     cx,130-8
        push    di

@@50:   stosb                           ; Draw one collumn
        add     di,X_Byte-1
        cmp     al,3+3                  ; Rotate collors
        jne     @@52
        mov     al,2
@@52:   inc     al
        loop    @@50

        pop     di                      ; Next collumn
        inc     di
        cmp     ah,3+3                  ; Rotate collors
        jne     @@54
        mov     ah,2
@@54:   inc     ah
        dec     dx
        jnz     @@40

        ret
EndP    Set_BG




; Load/create high score file (CY = Error)

Proc    Load_High

; Find/Make highscore name
        mov     si,Offset High_File     ; Assume default
        cmp     [Word ds:Param_Num],0
        je      @@20

        mov     si,Offset Param_Str     ; Convert to .hig extension
        push    si
        mov     al,'.'
@@10:   mov     ah,[Byte ds:si]         ; Get byte
        inc     si
        or      ah,ah                   ; Last char
        jz      @@15
        cmp     ah,al                   ; Dot?
        jne     @@10

@@15:   dec     si
        mov     [Word ds:si],'H.'
        mov     [Word ds:si+2],'GI'
        pop     si        

@@20:   mov     ax,06C00h               ; Open/Create file
        mov     bx,0010000000000010b    ; R/W acces, don't use INT 24h
        xor     cx,cx
        mov     dx,00010001b            ; Create if not exist
        int     21h
        jc      @@Err
        mov     [Word ds:High_Handle],ax ; Save handle

        cmp     cl,01                   ; Did file exist?
        je      @@25


; File was created, save default high score
        call    Save_High
        jmp     @@99


; File opend, now load it!        
@@25:   mov     bx,ax                   ; Handle
        mov     ah,03Fh
        mov     cx,33 + 14*6
        mov     dx,Offset High_List2
        int     21h
        jc      @@Err
        cmp     ax,cx
        jne     @@Err


; At last, decrypt it
        mov     si,Offset High_List
        mov     cx,(14*6) / 2
@@30:   mov     ax,[Word ds:si]
        ror     ax,5
        sub     ax,'GS'
        mov     [Word ds:si],ax
        add     si,2
        loop    @@30

@@99:   clc
@@Err:  ret
EndP    Load_High



; Save high score file (CY = Error)

Proc    Save_High

; Encrypt file and move to buffer
        mov     si,Offset High_List2
        mov     di,Offset Map1
        mov     cx,33                   ; Copy message
@@10:   mov     al,[Byte ds:si]
        inc     si
        mov     [Byte ds:di],al
        inc     di
        loop    @@10

        mov     cx,(14*6) / 2           ; Encrypt file
@@20:   mov     ax,[Word ds:si]
        add     si,2
        add     ax,'GS'
        rol     ax,5
        mov     [Word ds:di],ax
        add     di,2
        loop    @@20


; Seek to start of file
        mov     ax,04200h
        mov     bx,[Word ds:High_Handle]
        xor     cx,cx
        xor     dx,dx
        int     21h
        jc      @@Err


; Save file
        mov     ah,040h
        mov     cx,33 + 14*6
        mov     dx,Offset Map1
        int     21h
        jc      @@Err

@@99:   ret

@@Err:  add     sp,2*2                  ; Remove 2 words from the stack
        xor     al,al                   ; No 'file not found'
        jmp     Disk_Err
EndP    Save_High
